// ***************************************************************
// This file was created using the CreateProject.sh script
// for project HypothesisTest.
// CreateProject.sh is part of Bayesian Analysis Toolkit (BAT).
// BAT can be downloaded from http://www.mppmu.mpg.de/bat
// ***************************************************************

#include <BAT/BCLog.h>
#include <BAT/BCAux.h>
#include <BAT/BCSummaryTool.h>
#include <BAT/BCH1D.h> 

#include <TFile.h>
#include <TH1D.h>
#include <TCanvas.h>

#include "HypothesisTest.h"
#include  <BAT/BCTemplateFitter.h>
#include <BAT/BCModelManager.h>


int main()
{

   // ----------------------------------------------------
   // open file with data and templates
   // ----------------------------------------------------

   TFile * file = new TFile("histograms.root", "READ");

   TH1D hist_bkg1 = *((TH1D*) file->Get("hist_bkg1"));
   TH1D hist_bkg2 = *((TH1D*) file->Get("hist_bkg2"));
   TH1D hist_sgn  = *((TH1D*) file->Get("hist_sgn"));
   TH1D hist_data = *((TH1D*) file->Get("hist_data"));

   // close file
   file->Close();

   // set nicer style for drawing than the ROOT default
   BCAux::SetStyle();

   // open log file
   BCLog::OpenLog("log.txt");
   BCLog::SetLogLevel(BCLog::detail);

   // ----------------------------------------------------
   // model: 1 bkg 0 sgn
   // ----------------------------------------------------

   // create new BCTemplateFitter object
   BCTemplateFitter * m_1bkg0sgn = new BCTemplateFitter("model 1 bkg 0 sgn");

   // set data histogram
   m_1bkg0sgn->SetData(hist_data);

   // add template histograms
   m_1bkg0sgn->AddTemplate(hist_bkg1, "bkg 1", 40., 160.);

   // set prior
   m_1bkg0sgn->SetPriorConstant("bkg 1");

   // ----------------------------------------------------
   // model: 1 bkg 1 sgn
   // ----------------------------------------------------

   // create new BCTemplateFitter object
   BCTemplateFitter * m_1bkg1sgn = new BCTemplateFitter("model 1 bkg 1 sgn");

   // set data histogram
   m_1bkg1sgn->SetData(hist_data);

   // add template histograms
   m_1bkg1sgn->AddTemplate(hist_bkg1, "bkg 1", 40., 160.0);
   m_1bkg1sgn->AddTemplate(hist_sgn, "sgn", 0., 50.);

   // set priors
   m_1bkg1sgn->SetPriorConstant("bkg 1");
   m_1bkg1sgn->SetPriorConstant("sgn");

   // ----------------------------------------------------
   // model: 2 bkg 0 sgn
   // ----------------------------------------------------

   // create new BCTemplateFitter object
   BCTemplateFitter * m_2bkg0sgn = new BCTemplateFitter("model 2 bkg 0 sgn");

   // set data histogram
   m_2bkg0sgn->SetData(hist_data);

   // add template histograms
   m_2bkg0sgn->AddTemplate(hist_bkg1, "bkg 1", 40., 160.0);
   m_2bkg0sgn->AddTemplate(hist_bkg2, "bkg 2", 40., 160.0);

   // set priors
   m_2bkg0sgn->SetPriorConstant("bkg 1");
   m_2bkg0sgn->SetPriorConstant("bkg 2");

   // ----------------------------------------------------
   // model: 2 bkg 1 sgn
   // ----------------------------------------------------

   // create new BCTemplateFitter object
   BCTemplateFitter * m_2bkg1sgn = new BCTemplateFitter("model 2 bkg 1 sgn");
   m_2bkg1sgn->MCMCSetNLag(10);

   // set data histogram
   m_2bkg1sgn->SetData(hist_data);

   // add template histograms
   m_2bkg1sgn->AddTemplate(hist_bkg1, "bkg 1", 40., 160.);
   m_2bkg1sgn->AddTemplate(hist_bkg2, "bkg 2", 40., 160.);
   m_2bkg1sgn->AddTemplate(hist_sgn, "sgn", 0., 50.);

   // set priors
   m_2bkg1sgn->SetPriorConstant("bkg 1");
   m_2bkg1sgn->SetPriorConstant("bkg 2");
   m_2bkg1sgn->SetPriorConstant("sgn");


   // ----------------------------------------------------
   // perform analyses
   // ----------------------------------------------------

   // initialize models
   m_1bkg0sgn->Initialize();
   m_1bkg1sgn->Initialize();
   m_2bkg0sgn->Initialize();
   m_2bkg1sgn->Initialize();

   // perform marginalization and find global mode
   m_1bkg0sgn->MarginalizeAll();
   m_1bkg0sgn->FindMode(); 
   m_1bkg1sgn->MarginalizeAll();
   m_1bkg1sgn->FindMode(); 
   m_2bkg0sgn->MarginalizeAll();
   m_2bkg0sgn->FindMode();
   m_2bkg1sgn->MarginalizeAll();
   m_2bkg1sgn->FindMode();

   // calculate gof-variables
   double chi2_1bkg0sgn     = m_1bkg0sgn->CalculateChi2(m_1bkg0sgn->GetBestFitParameters());
   int ndf_1bkg0sgn         = m_1bkg0sgn->GetNDF();
   double chi2prob_1bkg0sgn = m_1bkg0sgn->CalculateChi2Prob(m_1bkg0sgn->GetBestFitParameters());
   double pvalue_1bkg0sgn   = m_1bkg0sgn->CalculatePValue();
   double KSprob_1bkg0sgn   = m_1bkg0sgn->CalculateKSProb();



   double chi2_1bkg1sgn = m_1bkg1sgn->CalculateChi2(m_1bkg1sgn->GetBestFitParameters());
   int ndf_1bkg1sgn     = m_1bkg1sgn->GetNDF();
   double chi2prob_1bkg1sgn = m_1bkg1sgn->CalculateChi2Prob(m_1bkg1sgn->GetBestFitParameters());
   double pvalue_1bkg1sgn   = m_1bkg1sgn->CalculatePValue();
   double KSprob_1bkg1sgn   = m_1bkg1sgn->CalculateKSProb();


   double chi2_2bkg0sgn =
m_2bkg0sgn->CalculateChi2(m_2bkg0sgn->GetBestFitParameters());
   int ndf_2bkg0sgn     = m_2bkg0sgn->GetNDF();
   double chi2prob_2bkg0sgn =
m_2bkg0sgn->CalculateChi2Prob(m_2bkg0sgn->GetBestFitParameters());
   double pvalue_2bkg0sgn   = m_2bkg0sgn->CalculatePValue();
   double KSprob_2bkg0sgn   = m_2bkg0sgn->CalculateKSProb();

   double chi2_2bkg1sgn =
m_2bkg1sgn->CalculateChi2(m_2bkg1sgn->GetBestFitParameters());
   int ndf_2bkg1sgn     = m_2bkg1sgn->GetNDF();
   double chi2prob_2bkg1sgn =
m_2bkg1sgn->CalculateChi2Prob(m_2bkg1sgn->GetBestFitParameters());
   double pvalue_2bkg1sgn   = m_2bkg1sgn->CalculatePValue();
   double KSprob_2bkg1sgn   = m_2bkg1sgn->CalculateKSProb();


   // ----------------------------------------------------
   // set up model manager
   // ----------------------------------------------------

   // create new BCTemplateFitterManager
   BCModelManager * smm = new BCModelManager();
   smm->AddModel(m_1bkg0sgn);
   smm->AddModel(m_1bkg1sgn);
   smm->AddModel(m_2bkg0sgn);
   smm->AddModel(m_2bkg1sgn);

   // Calculates the normalization of the likelihood for each model

   smm->Normalize();

   // compare models
   double bayesFact01 = smm->BayesFactor(0,1);

   std::cout << std::endl << "Bayes Factor 1bkg0sig/1bkg1sig = " << bayesFact01 << std::endl;
   



   // print gof-variables
   std::cout << std::endl << "Goodness-of-fit tests: " << std::endl;
   std::cout << " chi2 / dof (chi2 prob.)  = " << chi2_1bkg0sgn << " / " << ndf_1bkg0sgn
   << " (" << chi2prob_1bkg0sgn << ")" << std::endl;
   std::cout << " p-value                  = " << pvalue_1bkg0sgn << std::endl;
   std::cout << " Kolmogorov-Smirnov prob. = " << KSprob_1bkg0sgn << std::endl << std::endl; 


   // print gof-variables
   std::cout << std::endl << "Goodness-of-fit tests:  1bkg 1 sgn" << std::endl;
   std::cout << " chi2 / dof (chi2 prob.)  = " << chi2_1bkg1sgn << " / "
        << ndf_1bkg1sgn << " (" << chi2prob_1bkg1sgn << ")" << std::endl;
   std::cout << " p-value                  = " << pvalue_1bkg1sgn << std::endl;
   std::cout << " Kolmogorov-Smirnov prob. = " << KSprob_1bkg1sgn << std::endl << std::endl;


   // print gof-variables


   std::cout << std::endl << "Goodness-of-fit tests:  2bkg 0 sgn" << std::endl;
   std::cout << " chi2 / dof (chi2 prob.)  = " << chi2_2bkg0sgn << " / "
             << ndf_2bkg0sgn << " (" << chi2prob_2bkg0sgn << ")" << std::endl;
   std::cout << " p-value                  = " << pvalue_2bkg0sgn << std::endl;
   std::cout << " Kolmogorov-Smirnov prob. = " << KSprob_2bkg0sgn << std::endl <<
std::endl;

   // print gof-variables

   std::cout << std::endl << "Goodness-of-fit tests:  2bkg 1 sgn" << std::endl;
   std::cout << " chi2 / dof (chi2 prob.)  = " << chi2_2bkg1sgn << " / "
             << ndf_2bkg1sgn << " (" << chi2prob_2bkg1sgn << ")" << std::endl;
   std::cout << " p-value                  = " << pvalue_2bkg1sgn << std::endl;
   std::cout << " Kolmogorov-Smirnov prob. = " << KSprob_2bkg1sgn << std::endl <<
std::endl;



   // create new HypothesisTest object
   HypothesisTest * m = new HypothesisTest();

   BCLog::OutSummary("Test model created");

   // create a new summary tool object
   BCSummaryTool * summary = new BCSummaryTool(m);

   // perform your analysis here

   // normalize the posterior, i.e. integrate posterior
   // over the full parameter space
//   m->Normalize();

   // run MCMC and marginalize posterior wrt. all parameters
   // and all combinations of two parameters
//   m->MarginalizeAll();

   // run mode finding; by default using Minuit
//   m->FindMode();

   // if MCMC was run before (MarginalizeAll()) it is
   // possible to use the mode found by MCMC as
   // starting point of Minuit minimization
//   m->FindMode( m->GetBestFitParameters() );

   // draw all marginalized distributions into a PostScript file
//   m->PrintAllMarginalized("HypothesisTest_plots.ps");

   // print all summary plots
//   summary->PrintParameterPlot("HypothesisTest_parameters.eps");
//   summary->PrintCorrelationPlot("HypothesisTest_correlation.eps");
//   summary->PrintKnowledgeUpdatePlots("HypothesisTest_update.ps");

   // calculate p-value
//   m->CalculatePValue( m->GetBestFitParameters() );

   // print results of the analysis into a text file
//   m->PrintResults("HypothesisTest_results.txt");


   // ----------------------------------------------------
   // print
   // ----------------------------------------------------

   // print data
   TCanvas c1("c1");
   c1.cd();
   hist_data.Draw();
   c1.Print("data.ps"); 

   // print marginalized distribution for signal
   c1.cd();
   m_1bkg1sgn->GetMarginalized("sgn")->Draw(0, -95);
   c1.Print("1bkg1sgn_sgn.ps");

  // print marginalized distribution for signal


   c1.cd();
   m_2bkg1sgn->GetMarginalized("sgn")->Draw(0, -95);
   c1.Print("2bkg1sgn_sgn.ps");


   // print stack plots and results
   m_1bkg0sgn->PrintStack("1bkg0sgn_stack.ps"); 
   m_1bkg1sgn->PrintStack("1bkg1sgn_stack.eps");
   m_2bkg0sgn->PrintStack("2bkg0sgn_stack.eps");
   m_2bkg1sgn->PrintStack("2bkg1sgn_stack.eps");


   delete m;
   delete summary;

   // delete models
   delete m_1bkg0sgn;
   delete m_1bkg1sgn;
   delete m_2bkg0sgn;
   delete m_2bkg1sgn;

   // delete model manager
   delete smm; 


   BCLog::OutSummary("Test program ran successfully");
   BCLog::OutSummary("Exiting");

   // close log file
   BCLog::CloseLog();

   return 0;

}

